package main

import (
	"embed"
	"html/template"
	"io"
	"net/http"
	"sort"

	"git.agecem.com/agecem/agecem-org/public"
	"github.com/labstack/echo/v4"
	"github.com/labstack/echo/v4/middleware"
)

// Types

type Template struct {
	templates *template.Template
}

// Variables

var embedFS embed.FS

// Functions

func init() {
	embedFS = public.GetEmbedFS()
}

func Execute() {
	e := echo.New()

	t := &Template{
		templates: template.Must(template.ParseFS(embedFS, "html/*.gohtml")),
	}

	e.Renderer = t

	e.Pre(middleware.RemoveTrailingSlash())

	// API Routes

	e.GET("/v1", handleV1)

	// HTML Routes

	e.GET("/", handleIndex)

	e.Logger.Fatal(e.Start(":8080"))
}

func (t *Template) Render(w io.Writer, name string, data interface{}, c echo.Context) error {
	return t.templates.ExecuteTemplate(w, name, data)
}

// API Handlers

// handleV1 affiche les routes accessibles.
// Les routes sont triées selon .Path, pour les rendre plus facilement navigables.
func handleV1(c echo.Context) error {
	routes := c.Echo().Routes()
	sort.Slice(routes, func(i, j int) bool { return routes[i].Path < routes[j].Path })
	return c.JSON(http.StatusOK, routes)
}

// HTML Handlers

func handleIndex(c echo.Context) error {
	return c.Render(http.StatusOK, "index-html", nil)
}
